/*
 * Copyright 2008 Dan Furie and Scott Linderman
 *
 * This file contains methods for the menu system to interact with the
 * camera.
 */

//---------------------EEPROM VARIABLES-------------------------------
uint16_t EEMEM e_flashTime = 0x01;          //useFlash = 1
uint16_t EEMEM e_numShots = 0x0001;       //numShots = 1
uint16_t EEMEM e_exposureTime = 0x0001;   //exposureTime = 1s
unsigned char EEMEM e_audioThresh = 0x7f;       //audioThresh = 127/255
unsigned char EEMEM e_accelThresh = 0x7f;       //accelThresh = 127/255
unsigned char EEMEM e_photoThresh = 0x7f;       //photoThresh = 127/255

//-------------------SHOT STATE VARS---------------------------------
#define WAITING 0
#define FLASH 1
#define BEGIN 2
#define SHOT_DONE 3
#define CLOSE 4
char shotState;

#define EXT_EXP 0
#define AUDIO 1
#define ACCEL 2
#define PHOTO 3
#define BETWEEN_SHOTS 4 //define a value for between shots so we can clear state
                        //after a shot has been taken

char shotType;	  
unsigned char globalThresh;

#define CAMPORT PORTC
#define FLASH_PIN 6
#define SHUTTER_PIN 7

//------------------TIMER VALS-------------------------------------
unsigned int shotStateTimer;
unsigned int eeTimer;

unsigned char getADCValue(void)
{
   //get AD Converter value
   unsigned char v = ADCH;
   //prepare for next reading
   ADCSRA=ADCSRA | (0x01 << 6);
   return v;
}

void setGenSettings(unsigned int flashTime, unsigned int numShots)
{
   eeprom_write_word(&e_flashTime, flashTime);
   eeprom_write_word(&e_numShots, numShots);
}

void setExtendedExposureSettings(unsigned int exposureTime)
{
   eeprom_write_word(&e_exposureTime, exposureTime);
}

void setAudioTriggerSettings(char threshold)
{
   eeprom_write_byte(&e_audioThresh, threshold);
}

void setAccelerometerSettings(char threshold)
{
   eeprom_write_byte(&e_accelThresh, threshold);
}

void setPhotoTriggerSettings(char threshold)
{
   eeprom_write_byte(&e_photoThresh, threshold);
}

unsigned int getFlashTime(void)
{
   return eeprom_read_word(&e_flashTime);
}

unsigned int getNumShots(void)
{
   return eeprom_read_word(&e_numShots);
}

unsigned int getExposureTime(void)
{
   return eeprom_read_word(&e_exposureTime);
}

unsigned char getAudioThresh(void)
{
   return eeprom_read_byte(&e_audioThresh);
}

unsigned char getAccelThresh(void)
{
   return eeprom_read_byte(&e_accelThresh);
}

unsigned char getPhotoThresh(void)
{
   return eeprom_read_byte(&e_photoThresh);
}

void takeExtendedExposure(void)
{
   /*
    * begin taking a photo in extended exposure mode
    */
  eeTimer = 10000;
  shotType = EXT_EXP;
  shotStateTimer = eeprom_read_word(&e_exposureTime);
  shotState = BEGIN;
}
void takeAudioTriggered(void)
{
   /*
    * begin taking a photo in audio triggered mode
    */
    shotType = AUDIO;
    shotStateTimer = 2000; //make sure shutter is open for at least 200ms before starting shot
    globalThresh = eeprom_read_byte(&e_audioThresh);
    shotState = BEGIN;
}
void takeAccelerometerTriggered(void)
{
   /*
    * begin taking a photo in accelerometer triggered mode
    */
    shotType = ACCEL;
    shotStateTimer = 2000; //make sure shutter is open for at least 200ms before starting shot
    globalThresh = eeprom_read_byte(&e_accelThresh);
    shotState = BEGIN;
}
void takePhotoTriggered(void)
{
   /*
    * begin taking a photo in photo triggered mode
    */
    shotType = PHOTO;
    shotStateTimer = 2000; //make sure shutter is open for at least 200ms before starting shot
    globalThresh = eeprom_read_byte(&e_photoThresh);
    shotState = BEGIN;
}

void checkShot(void)
{
	/*
	 * This should be called every time through the main loop. It checks
	 * whether the threshold has been exceeded. If it has, timers are set to 
	 * trigger the flash and open the shutter. This is accomplished via a 
	 * shot state machine.
	 */
	 switch (shotState)
	 {
	 case BEGIN:
	    //we want to clear any old adc val that may be left over in adch, so 
	    //start a new ad conversion
	    getADCValue();
	   shutter(1); // Open the shutter
	   if (shotStateTimer == 0)
	   {
	      if (shotType != BETWEEN_SHOTS) shotState = WAITING; // Go to where we wait for the trigger event
	   }
	   break;
	 case WAITING:
	   // Waiting for the ADC value to reach the threshold for taking a picture.
	    if (shotType == EXT_EXP)
	    {
	       if (shotStateTimer == 0) shotState = CLOSE;
	    }
	    else if (shotType == PHOTO)
	    {
		  //photo triggers if we go below a threshold
		  if (getADCValue() < globalThresh) 
			{
			  shotState = FLASH; // Move to the flash timer state
			  shotStateTimer = getFlashTime(); // Set the time for the flashtime.
			}
	    }
	    // in accelerometer or audio we need to exceed a threshold
	    else if ((shotType == AUDIO) || (shotType == ACCEL))
	    {
	     if (getADCValue() > globalThresh) 
		  {
			shotState = FLASH; // Move to the flash timer state
			shotStateTimer = getFlashTime(); // Set the time for the flashtime.
		  }
	    }
		break;
	 case FLASH:
	   // Turn on the flash for length flashTime.
	   flash(1);
	   if (shotStateTimer == 0) {
		 shotState = CLOSE;
	   }
	   break;
	 case CLOSE:
	   // Close the shutter and turn off the flash.
	   flash(0);
	   shutter(0);
	   shotState = SHOT_DONE;
	   break;
	 case SHOT_DONE:
	   shotType = BETWEEN_SHOTS;
	   break;
	 }
}

void flash(char OnOff) {
  if (OnOff) {
	sbi(CAMPORT,FLASH_PIN);
  } else {
	cbi(CAMPORT,FLASH_PIN);
  }
}

void shutter(char OnOff) {
  if (OnOff) {
	sbi(CAMPORT,SHUTTER_PIN);
  } else {
	cbi(CAMPORT,SHUTTER_PIN);
  }
}

void stopShot(void)
{
	/*
	 * Stop the shot by simply setting shot state to done and clearing state vars
	 */
    flash(0);
    shutter(0);
	 shotState = SHOT_DONE;
	 shotStateTimer = 0;
}

char getShotDone(void)
{
   /*
    * this method provides a way for the menu to know if the shot completed
    */
   if (shotState == SHOT_DONE) return 1;
   else return 0;
}

unsigned char getAudioVal(void)
{	  	 
   return getADCValue();
}
unsigned char getAccelerometerVal(void)
{
   return getADCValue();
}
unsigned char getPhotoVal(void)
{
   return getADCValue();
}


